@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 """
     sphinxit.lexemes
     ~~~~~~~~~~~~~~~~
@@ -10,10 +8,15 @@
     :license: BSD, see LICENSE for more details.
 """
 
-import itertools
-import collections
+from __future__ import unicode_literals
+from __future__ import absolute_import
+
+import six
+from six.moves import filter, map
+
 import re
-from exceptions import SphinxQLChainException, SphinxQLSyntaxException
+import collections
+from .exceptions import SphinxQLChainException, SphinxQLSyntaxException
 
 
 RESERVED_KEYWORDS = ('AND', 'AS', 'ASC', 'AVG', 'BEGIN', 'BETWEEN', 'BY', 'CALL', 'COLLATION', 'COMMIT',
@@ -26,11 +29,11 @@ RESERVED_KEYWORDS = ('AND', 'AS', 'ASC', 'AVG', 'BEGIN', 'BETWEEN', 'BY', 'CALL'
 
 class SXQLSelect(object):
     _validator_exception_msg = 'Attributes are not defined or defined improperly.'
-    _lex_string = u'SELECT {attrs}'
-    _joiner_string = u', '
+    _lex_string = 'SELECT {attrs}'
+    _joiner_string = ', '
 
     def __init__(self, init_attrs=None):
-        init_attrs = init_attrs or [u'*']
+        init_attrs = init_attrs or ['*']
         self._attrs = []
         self._modificators = []
         self._attrs.append(*self._clean_attrs(*init_attrs))
@@ -38,7 +41,7 @@ class SXQLSelect(object):
     @property
     def lex(self):
         if self._modificators:
-            map(self._attrs.append, [l.lex for l in self._modificators])
+            self._attrs.extend([l.lex for l in self._modificators])
 
         if not self._attrs:
             raise SphinxQLSyntaxException(self._validator_exception_msg)
@@ -47,23 +50,23 @@ class SXQLSelect(object):
 
     def _clean_attrs(self, *attrs):
         if self._is_modificators(*attrs):
-            return itertools.ifilter(lambda x: x not in self._modificators, attrs)
+            return filter(lambda x: x not in self._modificators, attrs)
         if self._is_attributes(*attrs):
-            return itertools.ifilter(lambda x: x not in self._attrs, attrs)
+            return filter(lambda x: x not in self._attrs, attrs)
 
     def _is_modificators(self, *attrs):
-        return not False in map(lambda x: isinstance(x, (SXQLORFilter, Count, Avg)), attrs)
+        return all(map(lambda x: isinstance(x, (SXQLORFilter, Count, Avg)), attrs))
 
     def _is_attributes(self, *attrs):
-        return not False in map(lambda x: isinstance(x, basestring), attrs)
+        return all(map(lambda x: isinstance(x, six.string_types), attrs))
 
     def __call__(self, *attrs):
         if self._is_modificators(*attrs):
-            map(self._modificators.append, self._clean_attrs(*attrs))
+            self._modificators.extend(self._clean_attrs(*attrs))
         elif self._is_attributes(*attrs):
-            map(self._attrs.append, self._clean_attrs(*attrs))
-            if u'*' in self._attrs:
-                self._attrs.remove(u'*')
+            self._attrs.extend(self._clean_attrs(*attrs))
+            if '*' in self._attrs:
+                self._attrs.remove('*')
         else:
             raise SphinxQLSyntaxException(self._validator_exception_msg)
 
@@ -77,8 +80,8 @@ class SXQLSelect(object):
 
 class SXQLFrom(object):
     _validator_exception_msg = 'Indexes are not defined or defined improperly.'
-    _lex_string = u'FROM {indexes}'
-    _joiner_string = u', '
+    _lex_string = 'FROM {indexes}'
+    _joiner_string = ', '
 
     def __init__(self):
         self._attrs = []
@@ -93,13 +96,13 @@ class SXQLFrom(object):
         return self._lex_string.format(indexes=joined_attrs)
 
     def _clean_attrs(self, attrs):
-        if not attrs or False in map(lambda x: isinstance(x, basestring), attrs):
+        if not attrs or not all(map(lambda x: isinstance(x, six.string_types), attrs)):
             raise SphinxQLSyntaxException(self._validator_exception_msg)
 
-        return itertools.ifilter(lambda x: x not in self._attrs, attrs)
+        return filter(lambda x: x not in self._attrs, attrs)
 
     def __call__(self, *attrs):
-        map(self._attrs.append, self._clean_attrs(attrs))
+        self._attrs.extend(self._clean_attrs(attrs))
         return self
 
     def __add__(self, other):
@@ -116,52 +119,54 @@ class CommonSXQLWhereMixin(object):
     _not_valid_range_msg = "Condition has to be a range of two integers. '{0}' is not."
     _attr_value_is_range_msg = 'Attribute value has to be an integer, not range.'
 
-    allowed_conditions_map = {'__eq': u'{a}={v}',
-                              '__neq': u'{a}!={v}',
-                              '__gt': u'{a}>{v}',
-                              '__gte': u'{a}>={v}',
-                              '__lt': u'{a}<{v}',
-                              '__lte': u'{a}<={v}',
-                              '__in': u'{a} IN ({v})',
-                              '__between': u'{a} BETWEEN {f_v} AND {s_v}',
+    allowed_conditions_map = {'__eq': '{a}={v}',
+                              '__neq': '{a}!={v}',
+                              '__gt': '{a}>{v}',
+                              '__gte': '{a}>={v}',
+                              '__lt': '{a}<{v}',
+                              '__lte': '{a}<={v}',
+                              '__in': '{a} IN ({v})',
+                              '__between': '{a} BETWEEN {f_v} AND {s_v}',
                               }
 
     def _clean_rendered_attrs(self, k_attr, v_attr):
-        if isinstance(v_attr, basestring):
+        if isinstance(v_attr, six.string_types):
             try:
                 v_attr = int(v_attr)
             except ValueError:
                 raise SphinxQLSyntaxException(self._attr_value_is_string_msg)
 
+        if (isinstance(v_attr, collections.Iterable)
+            and not all(map(lambda x: isinstance(x, six.integer_types), v_attr))):
+            try:
+                v_attr = map(int, v_attr)
+            except ValueError:
+                raise SphinxQLSyntaxException(self._not_integer_values_msg.format(v_attr))
+
         for ending in self.allowed_conditions_map.keys():
-            if k_attr.endswith(ending):
-                a = k_attr[:k_attr.rindex(ending)]
-                v = v_attr
+            if not k_attr.endswith(ending):
+                continue
 
-                if ending not in ('__between', '__in') and isinstance(v_attr, collections.Iterable):
-                    raise SphinxQLSyntaxException(self._attr_value_is_range_msg)
+            if ending not in ('__between', '__in') and isinstance(v_attr, collections.Iterable):
+                raise SphinxQLSyntaxException(self._attr_value_is_range_msg)
 
-                if ending in ('__between', '__in'):
-                    if not isinstance(v_attr, collections.Iterable):
-                        raise SphinxQLSyntaxException(self._not_iterable_values_msg.format(v_attr))
+            if ending in ('__between', '__in'):
+                if not isinstance(v_attr, collections.Iterable):
+                    raise SphinxQLSyntaxException(self._not_iterable_values_msg.format(v_attr))
 
-                    if (isinstance(v_attr, collections.Iterable)
-                        and False in map(lambda x: isinstance(x, (int, long)), v_attr)):
-                        try:
-                            v_attr = map(int, v_attr)
-                        except ValueError:
-                            raise SphinxQLSyntaxException(self._not_integer_values_msg.format(v_attr))
+            a = k_attr[:k_attr.rindex(ending)]
+            v = v_attr
 
-                if ending == '__between':
-                    if len(v_attr) != 2:
-                        raise SphinxQLSyntaxException(self._not_valid_range_msg.format(v_attr))
-                    f_v, s_v = v_attr
-                    return self.allowed_conditions_map[ending].format(a=a, f_v=f_v, s_v=s_v)
+            if ending == '__between':
+                if len(v_attr) != 2:
+                    raise SphinxQLSyntaxException(self._not_valid_range_msg.format(v_attr))
+                f_v, s_v = v_attr
+                return self.allowed_conditions_map[ending].format(a=a, f_v=f_v, s_v=s_v)
 
-                elif ending == '__in':
-                    v = ','.join(map(str, v_attr))
+            elif ending == '__in':
+                v = ','.join(map(str, v_attr))
 
-                return self.allowed_conditions_map[ending].format(a=a, v=v)
+            return self.allowed_conditions_map[ending].format(a=a, v=v)
 
         raise SphinxQLSyntaxException(self._not_correct_attr_msg.format(k_attr))
 
@@ -169,9 +174,10 @@ class CommonSXQLWhereMixin(object):
 class SXQLMatch(object):
     _validator_exception_msg = "The query has to be a string. '{0}' is not."
     _empty_exception_msg = 'No query defined for full-text search.'
-    _eqs_chars = ['@', '!', '^', '(', ')', '+', '~', '-', '|', '[', ']', '/', '<<', '=', '$', '*', '"']
-    _lex_string = u"MATCH('{query}')"
-    _joiner_string = u' '
+    _single_escape_chars = ["'", '+', '[', ']', '=', '*']
+    _double_escape_chars = ['@', '!', '^', '(', ')', '~', '-', '|', '/', '<<', '$', '"']
+    _lex_string = "MATCH('{query}')"
+    _joiner_string = ' '
 
     def __init__(self):
         self._attrs = []
@@ -184,13 +190,15 @@ class SXQLMatch(object):
         return lex
 
     def __call__(self, query, escape=True):
-        if not isinstance(query, basestring):
+        if not isinstance(query, six.string_types):
             raise SphinxQLSyntaxException(self._validator_exception_msg.format(query))
 
         if escape:
-            substract_chars_re = '|\\'.join(self._eqs_chars)
-            query = re.sub(substract_chars_re, lambda m: r'\\{0}'.format(m.group()), query)
-            query = re.sub("'", '', query)  # escaping single quote doesn`t work
+            single_escape_chars_re = '|\\'.join(self._single_escape_chars)
+            query = re.sub(single_escape_chars_re, lambda m: r'\{0}'.format(m.group()), query)
+
+            double_escape_chars_re = '|\\'.join(self._double_escape_chars)
+            query = re.sub(double_escape_chars_re, lambda m: r'\\{0}'.format(m.group()), query)
 
         self._attrs.append(query)
 
@@ -205,14 +213,14 @@ class SXQLMatch(object):
 class SXQLLimit(object):
     _unique_exception_msg = 'Only one LIMIT clause is allowed in the query.'
     _validator_exception_msg = 'OFFSET and LIMIT values are not defined or defined improperly.'
-    _lex_string = u"LIMIT {offset},{limit}"
+    _lex_string = "LIMIT {offset},{limit}"
 
     def __init__(self, offset=None, limit=None):
         if offset and limit:
-            self.offset, self.limit = self.clean_attrs((offset, limit))
+            self.offset, self.limit = self._clean_attrs((offset, limit))
         else:
             self.offset = 0
-            self.limit = 1000
+            self.limit = 100
 
         self._is_called_once = False
 
@@ -222,8 +230,15 @@ class SXQLLimit(object):
         return lex
 
     def _clean_attrs(self, attrs):
-        if False in map(lambda x: isinstance(x, (int, long)), attrs) or len(attrs) != 2:
+        if len(attrs) != 2:
             raise SphinxQLSyntaxException(self._validator_exception_msg)
+
+        if not all(map(lambda x: isinstance(x, six.integer_types), attrs)):
+            try:
+                attrs = map(int, attrs)
+            except ValueError:
+                raise SphinxQLSyntaxException(self._validator_exception_msg)
+
         return attrs
 
     def __call__(self, *attrs):
@@ -241,19 +256,15 @@ class SXQLLimit(object):
 class SXQLOrder(object):
     _attrs_validator_exception_msg = 'Attributes are not defined or defined improperly.'
     _direction_validator_exception_msg = 'Order direction can be ASC or DESC only.'
-    _lex_string = u'ORDER BY {clauses}'
-    _joiner_string = u', '
+    _lex_string = 'ORDER BY {clauses}'
+    _joiner_string = ', '
 
     def __init__(self):
         self._attrs = []
 
     @property
     def lex(self):
-        if not self._attrs:
-            raise SphinxQLSyntaxException(self._attrs_validator_exception_msg)
-
         lex = self._lex_string.format(clauses=self._joiner_string.join(self._attrs))
-
         return lex
 
     def _clean_attrs(self, attrs):
@@ -261,7 +272,7 @@ class SXQLOrder(object):
             raise SphinxQLSyntaxException(self._attrs_validator_exception_msg)
 
         attr, direction = attrs
-        if False in (isinstance(attr, basestring), isinstance(direction, basestring)):
+        if not all(map(lambda x: isinstance(x, six.string_types), (attr, direction))):
             raise SphinxQLSyntaxException(self._attrs_validator_exception_msg)
         if direction.upper() not in ['ASC', 'DESC']:
             raise SphinxQLSyntaxException(self._direction_validator_exception_msg)
@@ -270,7 +281,7 @@ class SXQLOrder(object):
 
     def __call__(self, *attrs):
         attr, direction = self._clean_attrs(attrs)
-        self._attrs.append(u'{attr} {direction}'.format(attr=attr, direction=direction.upper()))
+        self._attrs.append('{attr} {direction}'.format(attr=attr, direction=direction.upper()))
         return self
 
     def __add__(self, other):
@@ -283,7 +294,7 @@ class SXQLGroupBy(object):
     _unique_exception_msg = 'Only one GROUP BY clause is allowed in the query.'
     _attr_validator_exception_msg = "Attribute name has to be a string. {0} is not."
     _no_attr_validator_exception_msg = 'No attribute name defined.'
-    _lex_string = u'GROUP BY {attr}'
+    _lex_string = 'GROUP BY {attr}'
 
     def __init__(self):
         self._attr = None
@@ -291,18 +302,16 @@ class SXQLGroupBy(object):
 
     @property
     def lex(self):
-        if not self._attr:
-            raise SphinxQLSyntaxException(self._no_attr_validator_exception_msg)
         lex = self._lex_string.format(attr=self._attr)
         return lex
 
     def _clean_attr(self, attr):
         if not attr:
             raise SphinxQLSyntaxException(self._no_attr_validator_exception_msg)
-        elif len(attr) != 1:
+        if len(attr) != 1:
             raise SphinxQLSyntaxException(self._unique_exception_msg)
-        elif not isinstance(attr[0], basestring):
-            raise SphinxQLSyntaxException(self._attr_validator_exception_msg)
+        if not isinstance(attr[0], six.string_types):
+            raise SphinxQLSyntaxException(self._attr_validator_exception_msg.format(attr[0]))
 
         return attr[0]
 
@@ -323,7 +332,7 @@ class SXQLGroupBy(object):
 
 class SXQLWithinGroupOrderBy(SXQLOrder):
     _unique_exception_msg = 'Only one WITHIN GROUP ORDER BY clause is allowed in the query'
-    _lex_string = u'WITHIN GROUP ORDER BY {clauses}'
+    _lex_string = 'WITHIN GROUP ORDER BY {clauses}'
 
     def __init__(self):
         self._is_called_once = False
@@ -337,7 +346,7 @@ class SXQLWithinGroupOrderBy(SXQLOrder):
         return super(SXQLWithinGroupOrderBy, self).__call__(*attrs)
 
     def __add__(self, other):
-        if isinstance(other, (SXQLLimit)):
+        if isinstance(other, SXQLLimit):
             return other
         raise SphinxQLChainException()
 
@@ -345,27 +354,27 @@ class SXQLWithinGroupOrderBy(SXQLOrder):
 class SXQLWhere(object):
     _validator_exception_msg = 'WHERE clause is not defined.'
     _container_exception_msg = 'SXQLWhere container is for SXQLFilter and SXQLMatch instances only.'
-    _lex_string = u'WHERE {clauses}'
-    _joiner_string = u' AND '
+    _lex_string = 'WHERE {clauses}'
+    _joiner_string = ' AND '
 
     def __init__(self):
         self._attrs = []
 
     @property
     def lex(self):
-        if not self._attrs:
-            raise SphinxQLSyntaxException(self._validator_exception_msg)
-
         return self._lex_string.format(clauses=self._joiner_string.join([l.lex for l in self._attrs]))
 
     def _clean_attrs(self, attrs):
-        if False in map(lambda x: isinstance(x, (SXQLFilter, SXQLMatch)), attrs):
+        if not attrs:
+            raise SphinxQLSyntaxException(self._validator_exception_msg)
+
+        if not all(map(lambda x: isinstance(x, (SXQLFilter, SXQLMatch)), attrs)):
             raise SphinxQLSyntaxException(self._container_exception_msg)
 
-        return itertools.ifilter(lambda x: x not in self._attrs, attrs)
+        return filter(lambda x: x not in self._attrs, attrs)
 
     def __call__(self, *attrs):
-        map(self._attrs.append, self._clean_attrs(attrs))
+        self._attrs.extend(self._clean_attrs(attrs))
         return self
 
     def __add__(self, other):
@@ -375,8 +384,8 @@ class SXQLWhere(object):
 
 
 class SXQLORFilter(object):
-    _lex_string = u'{clauses} AS cnd'
-    _joiner_string = u' AND '
+    _lex_string = '{clauses} AS cnd'
+    _joiner_string = ' AND '
 
     def __init__(self):
         self._attrs = []
@@ -395,8 +404,8 @@ class SXQLORFilter(object):
 
 
 class SXQLFilter(CommonSXQLWhereMixin):
-    _lex_string = u'{clauses}'
-    _joiner_string = u' AND '
+    _lex_string = '{clauses}'
+    _joiner_string = ' AND '
 
     def __init__(self):
         self._attrs = set()
@@ -413,16 +422,16 @@ class SXQLFilter(CommonSXQLWhereMixin):
 
 
 class Q(CommonSXQLWhereMixin):
-    _validator_exception_msg = u'Empty Q expression is not allowed.'
-    _lex_string = u'({clauses})'
-    _and_joiner = u' AND '
-    _or_joiner = u' OR '
+    _validator_exception_msg = 'Empty Q expression is not allowed.'
+    _lex_string = '({clauses})'
+    _and_joiner = ' AND '
+    _or_joiner = ' OR '
 
-    allowed_conditions_map = {'__eq': u'{a}={v}',
-                              '__gt': u'{a}>{v}',
-                              '__gte': u'{a}>={v}',
-                              '__lt': u'{a}<{v}',
-                              '__lte': u'{a}<={v}',
+    allowed_conditions_map = {'__eq': '{a}={v}',
+                              '__gt': '{a}>{v}',
+                              '__gte': '{a}>={v}',
+                              '__lt': '{a}<{v}',
+                              '__lte': '{a}<={v}',
                               }
 
     def __init__(self, **kwargs):
@@ -481,12 +490,12 @@ class AggregateObject(object):
             raise NotImplementedError('Can`t find some attributes')
 
     def _clean_attrs(self, attr, alias):
-        if not isinstance(attr, basestring):
+        if not isinstance(attr, six.string_types):
             raise SphinxQLSyntaxException(self._attr_validator_exception_msg.format(attr))
 
         alias = alias or self._default_alias_string.format(attr=attr)
 
-        if not isinstance(alias, basestring):
+        if not isinstance(alias, six.string_types):
             raise SphinxQLSyntaxException(self._alias_validator_exception_msg.format(alias))
         if alias.upper() in RESERVED_KEYWORDS:
             raise SphinxQLSyntaxException(self._alias_forbidden_exception_msg.format(alias))
@@ -501,29 +510,29 @@ class AggregateObject(object):
 
 
 class Avg(AggregateObject):
-    _lex_string = u'AVG({attr}) AS {alias}'
-    _default_alias_string = u'{attr}_avg'
+    _lex_string = 'AVG({attr}) AS {alias}'
+    _default_alias_string = '{attr}_avg'
 
 
 class Min(AggregateObject):
-    _lex_string = u'MIN({attr}) AS {alias}'
-    _default_alias_string = u'{attr}_min'
+    _lex_string = 'MIN({attr}) AS {alias}'
+    _default_alias_string = '{attr}_min'
 
 
 class Max(AggregateObject):
-    _lex_string = u'MAX({attr}) AS {alias}'
-    _default_alias_string = u'{attr}_max'
+    _lex_string = 'MAX({attr}) AS {alias}'
+    _default_alias_string = '{attr}_max'
 
 
 class Sum(AggregateObject):
-    _lex_string = u'SUM({attr}) AS {alias}'
-    _default_alias_string = u'{attr}_sum'
+    _lex_string = 'SUM({attr}) AS {alias}'
+    _default_alias_string = '{attr}_sum'
 
 
 class Count(AggregateObject):
-    _attr_lex_string = u'COUNT(DISTINCT {attr}) AS {alias}'
-    _star_lex_string = u'COUNT({attr}) AS {alias}'
-    _default_alias_string = u'{attr}_count'
+    _attr_lex_string = 'COUNT(DISTINCT {attr}) AS {alias}'
+    _star_lex_string = 'COUNT({attr}) AS {alias}'
+    _default_alias_string = '{attr}_count'
 
     @property
     def _lex_string(self):
@@ -533,16 +542,16 @@ class Count(AggregateObject):
         if attr and not alias:
             alias = self._default_alias_string.format(attr=attr)
         elif not attr and not alias:
-            alias = u'num'
-        attr = attr or u'*'
+            alias = 'num'
+        attr = attr or '*'
 
         return super(Count, self)._clean_attrs(attr, alias)
 
 
 class SXQLSnippets(object):
-    _default_lex_string = u"CALL SNIPPETS(({data}), '{index}', '{query}')"
-    _option_lex_string = u"CALL SNIPPETS(({data}), '{index}', '{query}', {options})"
-    _joiner_string = u', '
+    _default_lex_string = "CALL SNIPPETS(({data}), '{index}', '{query}')"
+    _option_lex_string = "CALL SNIPPETS(({data}), '{index}', '{query}', {options})"
+    _joiner_string = ', '
 
     _type_validator_exception_msg = "Wrong value for '{0}' snippet parameter is used. RTFM, please."
     _value_validator_exception_msg = "Wrong value for '{0}' snippet parameter is used. Allowed values are {1}."
@@ -550,11 +559,11 @@ class SXQLSnippets(object):
     _not_string_validator_exception_msg = "'{0}' attribute has to be a string."
     _data_validator_exception_msg = "Source data has to be a string or a list of strings. '{0} is not.'"
 
-    excerpts_params = {"before_match": {'type': basestring, 'defaults': '<b>'},
-                       "after_match": {'type': basestring, 'defaults': '</b>'},
-                       "chunk_separator": {'type': basestring, 'defaults': '...'},
-                       "limit": {'type': int, 'defaults': 256},
-                       "around": {'type': int, 'defaults': 5},
+    excerpts_params = {"before_match": {'type': six.string_types, 'defaults': '<b>'},
+                       "after_match": {'type': six.string_types, 'defaults': '</b>'},
+                       "chunk_separator": {'type': six.string_types, 'defaults': '...'},
+                       "limit": {'type': six.integer_types, 'defaults': 256},
+                       "around": {'type': six.integer_types, 'defaults': 5},
                        "exact_phrase": {'type': bool, 'defaults': 0},
                        "single_passage": {'type': bool, 'defaults': 0},
                        "use_boundaries": {'type': bool, 'defaults': 0},
@@ -563,21 +572,21 @@ class SXQLSnippets(object):
                        "force_all_words": {'type': bool, 'defaults': 0},
                        "limit_passages": {'type': bool, 'defaults': 0},
                        "limit_words": {'type': bool, 'defaults': 0},
-                       "start_passage_id": {'type': int, 'defaults': 1},
+                       "start_passage_id": {'type': six.integer_types, 'defaults': 1},
                        "load_files": {'type': bool, 'defaults': 0},
                        "load_files_scattered": {'type': bool, 'defaults': 0},
-                       "html_strip_mode": {'type': basestring,
+                       "html_strip_mode": {'type': six.string_types,
                                            'defaults': 'index',
                                            'allowed': ("none", "strip", "index", "retain")},
                        "allow_empty": {'type': bool, 'defaults': 0},
-                       "passage_boundary": {'type': basestring,
+                       "passage_boundary": {'type': six.string_types,
                                             'defaults': 'sentence',
                                             'allowed': ("sentence", "paragraph", "zone")},
                        "emit_zones": {'type': bool, 'defaults': 0},
                        }
 
-    type_to_str_map = {basestring: "'{0}' AS {1}",
-                       int: "{0} AS {1}",
+    type_to_str_map = {six.string_types: "'{0}' AS {1}",
+                       six.integer_types: "{0} AS {1}",
                        bool: "{0} AS {1}",
                        }
 
@@ -592,46 +601,51 @@ class SXQLSnippets(object):
         return self._option_lex_string if self._options else self._default_lex_string
 
     def _clean_attr(self, attr):
-        if not isinstance(attr, basestring):
+        if not isinstance(attr, six.string_types):
             raise SphinxQLSyntaxException(self._not_string_validator_exception_msg.format(attr))
 
-        attr = re.sub("'", '', attr)  # escaping single quote doesn`t work
+        attr = re.sub("'", lambda m: r'\{0}'.format(m.group()), attr)
 
         return attr
 
     def _clean_data(self, data):
         if not isinstance(data, collections.Iterable):
             raise SphinxQLSyntaxException(self._data_validator_exception_msg.format(data))
-        if not isinstance(data, basestring) and False in map(lambda d: isinstance(d, basestring), data):
+        if (not isinstance(data, six.string_types)
+            and False in map(lambda d: isinstance(d, six.string_types), data)):
             raise SphinxQLSyntaxException(self._data_validator_exception_msg.format(data))
-        if isinstance(data, basestring):
+        if isinstance(data, six.string_types):
             data = [data]
 
-        return map(lambda x: re.sub("'", '', x), data)  # escaping single quote doesn`t work
+        return map(lambda d: re.sub("'", lambda c: r'\{0}'.format(c.group()), d), data)
 
     def _clean_options(self, options):
         params_chain = []
         for k, v in options.items():
-            if k in self.excerpts_params.keys():
-                param_info = self.excerpts_params[k]
-                if not isinstance(v, param_info['type']):
-                    raise SphinxQLSyntaxException(self._type_validator_exception_msg.format(k))
-                if param_info.get('allowed', False) and v not in param_info['allowed']:
-                    raise SphinxQLSyntaxException(self._value_validator_exception_msg.format(
-                        k, self._joiner_string.join(["'{0}'".format(s) for s in param_info['allowed']]))
-                    )
-                if isinstance(v, bool):
-                    v = 1 if True else 0
-                param_string = self.type_to_str_map[param_info['type']]
-                params_chain.append(param_string.format(v, k))
-            else:
+            if k not in self.excerpts_params.keys():
                 raise SphinxQLSyntaxException(self._parameter_validator_exception_msg.format(k))
 
+            param_info = self.excerpts_params[k]
+            if not isinstance(v, param_info['type']):
+                raise SphinxQLSyntaxException(self._type_validator_exception_msg.format(k))
+
+            if param_info.get('allowed', False) and v not in param_info['allowed']:
+                raise SphinxQLSyntaxException(
+                    self._value_validator_exception_msg.format(
+                        k, self._joiner_string.join(["'{0}'".format(s) for s in param_info['allowed']]),
+                    )
+                )
+            if isinstance(v, bool):
+                v = 1 if v else 0
+
+            param_string = self.type_to_str_map[param_info['type']]
+            params_chain.append(param_string.format(v, k))
+
         return self._joiner_string.join(params_chain)
 
     @property
     def lex(self):
-        data = self._joiner_string.join([u"'{0}'".format(s) for s in self._clean_data(self._data)])
+        data = self._joiner_string.join(["'{0}'".format(s) for s in self._clean_data(self._data)])
         index = self._clean_attr(self._index)
         query = self._clean_attr(self._query)
 
